import { Component, OnInit, ElementRef } from '@angular/core';
import { ucapAnimations, DialogService } from '@ucap-webmessenger/ui';
import { NGXLogger } from 'ngx-logger';
import { Subscription, merge } from 'rxjs';
import { Store, select } from '@ngrx/store';

import * as AppStore from '@app/store';
import * as ChatStore from '@app/store/messenger/chat';
import * as SyncStore from '@app/store/messenger/sync';
import { UserInfo } from '@ucap-webmessenger/protocol-sync';
import {
  UserInfoSS,
  UserInfoF,
  UserInfoDN
} from '@ucap-webmessenger/protocol-query';
import { LoginResponse } from '@ucap-webmessenger/protocol-authentication';
import { SessionStorageService } from '@ucap-webmessenger/web-storage';

import { TranslateService } from '@ngx-translate/core';

import {
  OrganizationContactDialogComponent,
  OrganizationContactDialogData,
  OrganizationContactDialogResult
} from '@app/layouts/messenger/dialogs/call/organization-contact.dialog.component';
import {
  PhoneBookDialogComponent,
  PhoneBookDialogData,
  PhoneBookDialogResult
} from '@app/layouts/messenger/dialogs/call/phone-book.dialog.component';
import {
  CallHistoryDialogComponent,
  CallHistoryDialogData,
  CallHistoryDialogResult
} from '@app/layouts/messenger/dialogs/call/call-history.dialog.component';

@Component({
  selector: 'app-layout-chat-left-sidenav-call',
  templateUrl: './call.component.html',
  styleUrls: ['./call.component.scss'],
  animations: ucapAnimations
})
export class CallComponent implements OnInit {
  constructor(
    private store: Store<any>,
    public elementRef: ElementRef<HTMLElement>,
    private dialogService: DialogService,
    private sessionStorageService: SessionStorageService,
    private translateService: TranslateService,
    private logger: NGXLogger
  ) {}

  ngOnInit() {}

  async onClickCallHistory() {
    const result = await this.dialogService.open<
      CallHistoryDialogComponent,
      CallHistoryDialogData,
      CallHistoryDialogResult
    >(CallHistoryDialogComponent, {
      width: '700px',
      height: '600px',
      data: {}
    });

    if (!!result && !!result.choice && result.choice) {
      this.logger.debug('');
    }
  }

  async onClickOrgContact() {
    const result = await this.dialogService.open<
      OrganizationContactDialogComponent,
      OrganizationContactDialogData,
      OrganizationContactDialogResult
    >(OrganizationContactDialogComponent, {
      width: '700px',
      data: {}
    });

    if (!!result && !!result.choice && result.choice) {
      this.logger.debug('');
    }
  }

  async onClickPhoneBook() {
    const result = await this.dialogService.open<
      PhoneBookDialogComponent,
      PhoneBookDialogData,
      PhoneBookDialogResult
    >(PhoneBookDialogComponent, {
      width: '700px',
      data: {}
    });

    if (!!result && !!result.choice && result.choice) {
      this.logger.debug('');
    }
  }
}
