import {
  Component,
  OnInit,
  Inject,
  ElementRef,
  AfterViewInit
} from '@angular/core';
import { MatDialogRef, MAT_DIALOG_DATA } from '@angular/material';
import { EnvironmentsInfo, KEY_ENVIRONMENTS_INFO } from '@app/types';
import { SessionStorageService } from '@ucap-webmessenger/web-storage';
import { NGXLogger } from 'ngx-logger';
import {
  Observable,
  combineLatest,
  Subscription,
  of,
  BehaviorSubject
} from 'rxjs';
import { map, tap, catchError, take } from 'rxjs/operators';
import { Store, select } from '@ngrx/store';
import * as AppStore from '@app/store';
import { UCAP_NATIVE_SERVICE, NativeService } from '@ucap-webmessenger/native';
import { LoginResponse } from '@ucap-webmessenger/protocol-authentication';
import {
  PiService,
  CallHistoryResponse,
  CallHistoryInfo
} from '@ucap-webmessenger/pi';

export interface CallHistoryDialogData {}

// tslint:disable-next-line: no-empty-interface
export interface CallHistoryDialogResult {
  choice: boolean;
  phoneNumber: string;
}

@Component({
  selector: 'app-layout-messenger-call-history',
  templateUrl: './call-history.dialog.component.html',
  styleUrls: ['./call-history.dialog.component.scss']
})
export class CallHistoryDialogComponent implements OnInit, AfterViewInit {
  constructor(
    public dialogRef: MatDialogRef<
      CallHistoryDialogData,
      CallHistoryDialogResult
    >,
    private store: Store<any>,
    @Inject(MAT_DIALOG_DATA) public data: CallHistoryDialogData,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    private sessionStorageService: SessionStorageService,
    private elementRef: ElementRef,
    private piService: PiService,
    private logger: NGXLogger
  ) {
    this.environmentsInfo = this.sessionStorageService.get<EnvironmentsInfo>(
      KEY_ENVIRONMENTS_INFO
    );
  }
  environmentsInfo: EnvironmentsInfo;

  loginRes: LoginResponse;
  loginResSubscription: Subscription;

  callHistoryRes: CallHistoryResponse;
  callHistoryInfos: CallHistoryInfo[];

  ngOnInit() {
    this.loginResSubscription = this.store
      .pipe(
        select(AppStore.AccountSelector.AuthenticationSelector.loginRes),
        tap(loginRes => {
          this.loginRes = loginRes;
        })
      )
      .subscribe();

    this.piService
      .callHistory({
        userSeq: this.loginRes.userSeq,
        deviceType: this.environmentsInfo.deviceType,
        tokenKey: this.loginRes.tokenString
      })
      .pipe(
        take(1),
        map(res => {
          // this.logger.debug(res);
          this.callHistoryRes = res;
          const callDataList = [] as CallHistoryInfo[];
          res.data.forEach(v => {
            callDataList.push(v);
          });
          this.callHistoryInfos = callDataList;
        }),
        catchError(error => of(this.logger.debug(error)))
      )
      .subscribe();
  }

  ngAfterViewInit(): void {
    if (
      !!this.elementRef.nativeElement &&
      !!this.elementRef.nativeElement.querySelector('a')
    ) {
      const elements = this.elementRef.nativeElement.querySelectorAll('a');
      elements.forEach(element => {
        element.addEventListener('click', this.onClickEvent.bind(this));
      });
    }
  }

  onClickEvent(event: MouseEvent) {
    this.nativeService.openDefaultBrowser(
      (event.target as HTMLAnchorElement).text
    );
  }

  onClickChoice() {
    this.dialogRef.close();
  }
}
