import { Component, OnInit, OnDestroy, Inject } from '@angular/core';

import { Store, select } from '@ngrx/store';

import { Company } from '@ucap-webmessenger/api-external';
import { ServerErrorCode, ProtocolService } from '@ucap-webmessenger/protocol';

import * as AppStore from '@app/store';
import * as AuthenticationStore from '@app/store/account/authentication';
import * as CompanyStore from '@app/store/setting/company';
import { Observable, Subscription } from 'rxjs';
import { Router } from '@angular/router';
import { tap, map } from 'rxjs/operators';
import {
  DialogService,
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult
} from '@ucap-webmessenger/ui';
import { StringUtil } from '@ucap-webmessenger/core';
import {
  NoticeDialogComponent,
  NoticeDialogResult,
  NoticeDialogData
} from '@app/layouts/messenger/dialogs/account/notice.dialog.component';

import { environment } from '../../../../environments/environment';
import {
  LocalStorageService,
  SessionStorageService
} from '@ucap-webmessenger/web-storage';
import { AppUserInfo, KEY_APP_USER_INFO } from '@app/types/app-user-info.type';
import { LogoutInfo, KEY_LOGOUT_INFO } from '@app/types';
import { AppAuthenticationService } from '@app/services/authentication.service';
import { logoutInitialize } from '@app/store/account/authentication';
import { TranslateService } from '@ngx-translate/core';
import { NativeService, UCAP_NATIVE_SERVICE } from '@ucap-webmessenger/native';

@Component({
  selector: 'app-page-account-login',
  templateUrl: './login.page.component.html',
  styleUrls: ['./login.page.component.scss']
})
export class LoginPageComponent implements OnInit, OnDestroy {
  fixedCompany: string;
  fixedNotiBtnText: string;

  companyList$: Observable<Company[]>;

  loginFailureCount: Subscription;

  defatulLoginBtnText: string;
  loginBtnText: string;
  loginBtnEnable: boolean;

  timeChecker: any;
  defatulWaitingTime: number;
  waitingTime: number;

  appUserInfo: AppUserInfo;
  useRememberMe: boolean;
  useAutoLogin: boolean;

  rotateInfomationIntervalObject: any;
  rotateInfomationIndex = 0;
  rotateInfomation = [
    {
      index: 0,
      content: `우리는 브랜드를 통해 고객의 삶을 향상시킨다.`,
      info: `LF는 사람들의 라이프스타일을 읽고 트렌드를 선도하며 착의하는패션 제작에서 함께 누리고 즐기는 삶 속의 패션 문화 창조에 이르기까지 기업의 임무를 발전시켜 왔습니다.`
    },
    {
      index: 1,
      content: `LIFESTYLE BRAND COMPANY `,
      info: `LF는 다양한 라이프스타일을 접목하고 융합하여  글로벌 기준의 일류 브랜드를육성할 것입니다.최고 수준의 브낸드로 최사의 가치를 고객에게 드리기 위하여 LF는 세 가지   핵심 역량을 바탕으로 움직이고 있습니다.`
    },
    {
      index: 2,
      content: `LF 핵심 역량`,
      info: `첫째, 각 분야의 전문가이자 크리에이터로 임하고자 하는감각적인 Passion
              둘째, 정직과 공정으로 옷과 사람과 기업 간의 신뢰를 쌓아 온 굳건한 Ethic
              셋쩨, 최고를 만들기 위해 선도하는 Innovation `
    }
  ];

  constructor(
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    private store: Store<any>,
    private router: Router,
    private translateService: TranslateService,
    private dialogService: DialogService,
    private protocolService: ProtocolService,
    private localStorageService: LocalStorageService,
    private sessionStorageService: SessionStorageService,
    private appAuthenticationService: AppAuthenticationService
  ) {
    this.useRememberMe =
      environment.productConfig.authentication.rememberMe.use;
    this.useAutoLogin = environment.productConfig.authentication.autoLogin.use;

    this.appUserInfo = this.localStorageService.encGet<AppUserInfo>(
      KEY_APP_USER_INFO,
      environment.customConfig.appKey
    );

    this.rotateInfomationIndex =
      new Date().getTime() % this.rotateInfomation.length;
  }

  ngOnInit(): void {
    // this.appAuthenticationService.logout();
    this.protocolService.disconnect();
    this.store.dispatch(logoutInitialize());

    this.rotateInfomationIntervalObject = setInterval(() => {
      this.rotateInfomationIndex =
        (this.rotateInfomationIndex + 1) % this.rotateInfomation.length;
    }, 8000);

    this.defatulLoginBtnText = this.translateService.instant('accounts.login');
    this.defatulWaitingTime = 5 * 60; // sec

    this.store.dispatch(
      CompanyStore.companyList({
        companyGroupCode: environment.companyConfig.companyGroupCode
      })
    );

    this.companyList$ = this.store.pipe(
      select(AppStore.SettingSelector.CompanySelector.companyList)
    );

    this.loginFailureCount = this.store
      .pipe(
        select(AppStore.AccountSelector.AuthenticationSelector.loginFailCount),
        map(count => {
          if (count > 0) {
            if (count < 5) {
              this.dialogService.open<
                AlertDialogComponent,
                AlertDialogData,
                AlertDialogResult
              >(AlertDialogComponent, {
                width: '360px',
                data: {
                  title: '',
                  html: this.translateService.instant(
                    'accounts.errors.loginFailedIdPw'
                  )
                }
              });
            }

            if (count === 5) {
              this.dialogService.open<
                AlertDialogComponent,
                AlertDialogData,
                AlertDialogResult
              >(AlertDialogComponent, {
                width: '360px',
                data: {
                  title: '',
                  html: this.translateService.instant(
                    'accounts.errors.loginFailOverTry'
                  )
                }
              });

              this.timeChecker = setInterval(() => this.getCheckTime(), 1000);
            } else {
              this.loginBtnEnable = true;
            }
            return;
          } else {
            if (!!this.timeChecker) {
              clearInterval(this.timeChecker);
            }
            this.waitingTime = this.defatulWaitingTime;
            this.loginBtnText = this.defatulLoginBtnText;
            this.loginBtnEnable = true;
          }
        })
      )
      .subscribe();

    this.customInitilize();
  }

  async customInitilize() {
    const personLogout: LogoutInfo = this.sessionStorageService.get(
      KEY_LOGOUT_INFO
    );

    if (!!personLogout) {
      // if self logout clear idle check.
      this.nativeService.idleStateStop();
    }

    if (!!personLogout && !!personLogout.reasonCode) {
      let msg = this.translateService.instant('accounts.results.doLogout');
      switch (personLogout.reasonCode) {
        case ServerErrorCode.ERRCD_DUPLICATE:
          msg = this.translateService.instant(
            'accounts.results.doLogoutDuplicate',
            {
              ip: personLogout.ip,
              mac: personLogout.mac.replace(/:/g, '-')
            }
          );

          break;
        case ServerErrorCode.ERRCD_FORCE_INIT:
          msg = this.translateService.instant(
            'accounts.results.doLogoutByOthers'
          );
          break;
      }

      const result = this.dialogService.open<
        AlertDialogComponent,
        AlertDialogData,
        AlertDialogResult
      >(AlertDialogComponent, {
        width: '360px',
        data: {
          title: '',
          html: msg
        }
      });

      this.sessionStorageService.set<LogoutInfo>(KEY_LOGOUT_INFO, {
        personLogout: true
      } as LogoutInfo);
    }

    // Daesang..
    this.fixedCompany = environment.companyConfig.fixedCompanyCode;
    this.fixedNotiBtnText = this.translateService.instant(
      'accounts.usageNotes'
    );
  }

  ngOnDestroy(): void {
    if (!!this.loginFailureCount) {
      this.loginFailureCount.unsubscribe();
    }
  }

  getCheckTime() {
    if (this.waitingTime <= 0) {
      // reset.
      if (!!this.timeChecker) {
        clearInterval(this.timeChecker);
      }
      this.waitingTime = this.defatulWaitingTime;
      this.loginBtnText = this.defatulLoginBtnText;
      this.loginBtnEnable = true;
    } else {
      // wait.
      this.waitingTime = this.waitingTime - 1;

      this.loginBtnText = `${StringUtil.zeroFill(
        Math.floor(this.waitingTime / 60),
        2
      )}:${StringUtil.zeroFill(this.waitingTime % 60, 2)}`;
      this.loginBtnEnable = false;
    }
  }

  onLogin(value: {
    companyCode: string;
    loginId: string;
    loginPw: string;
    rememberMe: boolean;
    autoLogin: boolean;
    notValid: () => void;
  }) {
    this.loginBtnEnable = false;
    setTimeout(() => {
      this.loginBtnEnable = true;
    }, 30 * 1000);

    this.store.dispatch(
      AuthenticationStore.webLogin({
        loginInfo: {
          companyCode: value.companyCode,
          companyGroupType: 'C',
          loginId: value.loginId,
          loginPw: value.loginPw
        },
        rememberMe: value.rememberMe,
        autoLogin: value.autoLogin
      })
    );
  }

  onClickNoti() {
    // For Daesang,,
    this.dialogService.open<
      NoticeDialogComponent,
      NoticeDialogData,
      NoticeDialogResult
    >(NoticeDialogComponent, {
      width: '600px',
      data: {
        title: '이용시 주의사항',
        html: `
        <ol class='caution'>
          <li>메신저 계정/비밀번호를 타인에게 공유하지 않아야 합니다.</li>
          <li>회사 중요정보(고객정보 포함)를 업무상 필요한 인원에게 필요한 만큼만 공유해야 합니다.</li>
          <li>퇴근, 회의 등 자리를 비우는 경우 중요자료가 방치되지 않도록 주의하고, Clean Desk를 실천해야 합니다.</li>
          <li>메신저를 누군가 허락 없이 함부로 열람하고 그 내부 내용을 타인과 공유하지 않아야 합니다.</li>
          <li>공용PC에 메신저 사용 후 반드시 로그아웃하시기 바랍니다.</li>
          <li>사내메신저는 업무용 협업도구입니다. 비업무용 소통을 자제하시기 바랍니다.</li>
          <li>업무상 인지한 정보(개인의 프라이버시 포함)나 일반인에게 공개되지 않은 회사기록을 통해 얻은 정보는 오직 회사의 이익을 위해서 활용하여야 하며 사외에 유출하거나 타인에게 알려주거나 업무 이외의 목적에 사용하여서는 아니 됩니다.</li>
        </ol>`
      }
    });
  }

  onClickNextInfomation() {
    this.rotateInfomationIndex =
      (this.rotateInfomationIndex + 1) % this.rotateInfomation.length;

    if (!!this.rotateInfomationIntervalObject) {
      clearInterval(this.rotateInfomationIntervalObject);
      this.rotateInfomationIntervalObject = setInterval(() => {
        this.rotateInfomationIndex =
          (this.rotateInfomationIndex + 1) % this.rotateInfomation.length;
      }, 8000);
    }
  }

  onClickTemplate() {
    this.router.navigate(['/template']);
  }
}
