import {
  APIRequest,
  APIResponse,
  APIDecoder,
  APIEncoder,
  ParameterUtil,
  JsonAnalization
} from '@ucap-webmessenger/api';
import { DeviceType } from '@ucap-webmessenger/core';
import { PromptAPIResponse } from './api-prompt';
import { PIRequest, PIResponse } from './pi';
import { CallHistoryInfo } from '../models/call-history-info';
import { DialStatus } from '../types/dial-status.type';

const callHistorysJson = `{
  "page": 1,
  "page_size": 10,
  "total_count": 20,
  "data": [
    {
      "src_number": "0101234567",
      "dst_number": "0102342231",
      "status": "1",
      "req_date": "2019-12-20 12:22:00",
      "src_ans_time": "2019-12-20 12:22:10",
      "dst_ans_time": "2019-12-20 12:22:16",
      "end_time": "2019-12-20 12:23:00",
      "dial_status": "ANSWER|ANSWER"
    },
    {
      "src_number": "0101234567",
      "dst_number": "0102342231",
      "status": "1",
      "req_date": "2019-12-20 12:22:10",
      "src_ans_time": "2019-12-20 12:22:10",
      "dst_ans_time": "2019-12-20 12:22:16",
      "end_time": "2019-12-20 12:23:00",
      "dial_status": "ANSWER|ANSWER"
    },
    {
      "src_number": "0101234567",
      "dst_number": "0102342233",
      "status": "1",
      "req_date": "2019-12-20 12:22:20",
      "src_ans_time": "2019-12-20 12:22:10",
      "dst_ans_time": "2019-12-20 12:22:16",
      "end_time": "2019-12-20 12:23:00",
      "dial_status": "ANSWER|ANSWER"
    },
    {
      "src_number": "0101234567",
      "dst_number": "0102342233",
      "status": "1",
      "req_date": "2019-12-21 12:22:00",
      "src_ans_time": "2019-12-21 12:22:10",
      "dst_ans_time": "2019-12-21 12:22:16",
      "end_time": "2019-12-21 12:23:00",
      "dial_status": "ANSWER|ANSWER"
    },
    {
      "src_number": "0101234567",
      "dst_number": "0102342234",
      "status": "1",
      "req_date": "2019-12-21 12:22:00",
      "src_ans_time": "2019-12-21 12:22:10",
      "dst_ans_time": "2019-12-21 12:22:16",
      "end_time": "2019-12-21 12:23:00",
      "dial_status": "ANSWER|ANSWER"
    },
    {
      "src_number": "0101234567",
      "dst_number": "0102342235",
      "status": "1",
      "req_date": "2019-12-22 12:22:00",
      "src_ans_time": "2019-12-22 12:22:10",
      "dst_ans_time": "2019-12-22 12:22:16",
      "end_time": "2019-12-22 12:23:00",
      "dial_status": "ANSWER|ANSWER"
    },
    {
      "src_number": "0101234567",
      "dst_number": "0102342236",
      "status": "1",
      "req_date": "2019-12-22 12:22:00",
      "src_ans_time": "2019-12-22 12:22:10",
      "dst_ans_time": "2019-12-22 12:22:16",
      "end_time": "2019-12-22 12:23:00",
      "dial_status": "ANSWER|ANSWER"
    },
    {
      "src_number": "0101234567",
      "dst_number": "0102342238",
      "status": "1",
      "req_date": "2019-12-26 12:22:00",
      "src_ans_time": "2019-12-25 12:22:10",
      "dst_ans_time": "2019-12-25 12:22:16",
      "end_time": "2019-12-20 12:23:00",
      "dial_status": "ANSWER|ANSWER"
    },
    {
      "src_number": "0101234567",
      "dst_number": "0102342230",
      "status": "1",
      "req_date": "2019-12-26 12:22:00",
      "src_ans_time": "2019-12-25 12:22:10",
      "dst_ans_time": "2019-12-25 12:22:16",
      "end_time": "2019-12-25 12:23:00",
      "dial_status": "ANSWER|ANSWER"
    },
    {
      "src_number": "0101234567",
      "dst_number": "0102342230",
      "status": "1",
      "req_date": "2019-12-26 12:22:00",
      "src_ans_time": "2019-12-25 12:22:10",
      "dst_ans_time": "2019-12-25 12:22:16",
      "end_time": "2019-12-25 12:23:00",
      "dial_status": "ANSWER|ANSWER"
    },
    {
      "src_number": "0101234567",
      "dst_number": "0102342230",
      "status": "1",
      "req_date": "2019-12-26 12:22:00",
      "src_ans_time": "2019-12-25 12:22:10",
      "dst_ans_time": "2019-12-25 12:22:16",
      "end_time": "2019-12-25 12:23:00",
      "dial_status": "ANSWER|ANSWER"
    },
    {
      "src_number": "0101234567",
      "dst_number": "0102342230",
      "status": "1",
      "req_date": "2019-12-26 12:22:00",
      "src_ans_time": "2019-12-25 12:22:10",
      "dst_ans_time": "2019-12-25 12:22:16",
      "end_time": "2019-12-25 12:23:00",
      "dial_status": "ANSWER|ANSWER"
    },
    {
      "src_number": "0101234567",
      "dst_number": "0102342230",
      "status": "1",
      "req_date": "2019-12-26 12:22:00",
      "src_ans_time": "2019-12-25 12:22:10",
      "dst_ans_time": "2019-12-25 12:22:16",
      "end_time": "2019-12-25 12:23:00",
      "dial_status": "ANSWER|ANSWER"
    }
  ]
}`;

export interface CallRequest extends APIRequest {
  userSeq: number;
  deviceType: DeviceType;
  tokenKey: string;
  calleeNumber: string;
}

export interface CallResponse extends PromptAPIResponse {}

const callEncodeMap = {
  userSeq: 'userSeq',
  deviceType: 'deviceType',
  tokenKey: 'tokenKey',
  calleeNumber: 'calleeNumber'
};

export interface CallHistoryRequest extends APIRequest {
  userSeq: number;
  deviceType: DeviceType;
  tokenKey: string;
  page?: number;
  pageSize?: number;
}

export interface CallHistoryData extends APIResponse {
  callHistoryInfos: CallHistoryInfo[];
}
export interface CallHistoryResponse extends APIResponse {
  /** 페이지(n) */
  pageCurrent?: number;
  /** 전체갯수(n) */
  pageTotalCount: number;
  /** 요청리스트(n) */
  pageListCount?: number;
  data: CallHistoryInfo[];
  returnJson: string;
}

const callHistoryEncodeMap = {
  userSeq: 'userSeq',
  deviceType: 'deviceType',
  tokenKey: 'tokenKey',
  pageCurrent: 'page',
  pageListCount: 'pageSize'
};

export const encodeCall: APIEncoder<CallRequest> = (req: CallRequest) => {
  return ParameterUtil.encode(callEncodeMap, req);
};

export const decodeCall: APIDecoder<CallResponse> = (res: any) => {
  return {
    responseCode: res.responseCode,
    responseMsg: res.responseMsg
  } as CallResponse;
};

export const encodeCallHistory: APIEncoder<CallHistoryRequest> = (
  req: CallHistoryRequest
) => {
  return ParameterUtil.encode(callHistoryEncodeMap, req);
};

export const decodeCallHistory: APIDecoder<CallHistoryResponse> = (
  res: any
) => {
  const resLog = {} as CallHistoryResponse;
  const callHistoryInfos = [] as CallHistoryInfo[];

  const temp = String(callHistorysJson);
  const json = JSON.parse(temp);

  resLog.pageCurrent = json.page;
  resLog.pageTotalCount = json.total_count;
  resLog.pageListCount = json.page_size;

  json.data.forEach(v => {
    const dialSatusTemp = v.dial_status.split('|');

    callHistoryInfos.push({
      srcNumber: v.src_number,
      dstNumber: v.dst_number,
      status: v.status,
      reqDate: v.req_date,
      srcAnswerTime: v.src_ans_time,
      dstAnswerTime: v.dst_ans_time,
      endTime: v.end_time,
      srcDialStatus: dialSatusTemp[0] as DialStatus,
      dstDialStatus: dialSatusTemp[1] as DialStatus
    } as CallHistoryInfo);
  });
  resLog.data = callHistoryInfos;
  return resLog;
};
